PRECESSING_APPROXIMANTS = [
    "IMRPhenomXPHM",
    "IMRPhenomXPNR",
    "IMRPhenomTPHM",
    "IMRPhenomXPHM-SpinTaylor",
    "SEOBNRv5PHM",
    "NRSur7dq4",
]


ECCENTRIC_APPROXIMANTS = [
    "IMRPhenomTEHM",
    "IMRPhenomTE",
    "IMRPhenomXEHM",
    "IMRPhenomXE",
]

PRECESSING_PARAMETERS = [
    "chi_p",
    "spin1_x",
    "spin1_y",
    "spin2_x",
    "spin2_y",
    "phi_jl",
]

ECCENTRIC_PARAMETERS = [
    "eccentricity",
    "mean_anomaly",
]